<?php

namespace Drupal\mail_box_management\Twig\Extension;

use Drupal\mail_box_management\Server\Imap\Flag;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

class MailBoxManagementTwigExtension extends AbstractExtension {

  /**
   * Registers custom Twig functions.
   */
  public function getFunctions(): array {
    return [
      new TwigFunction('mail_box_color', [$this, 'getColor']),
      new TwigFunction('is_unseen', [$this, 'isUnseen']),
      new TwigFunction('is_flagged', [$this, 'isFlagged']),
      new TwigFunction('is_recent', [$this, 'isRecent']),
      new TwigFunction('json', [$this, 'json']),
      new TwigFunction('box_icon', [$this, 'getIcon']),
      new TwigFunction('attachment_size', [$this, 'getSize']),
      new TwigFunction('mail_actions', [$this, 'getActions']),
      new TwigFunction('mail_strip', [$this, 'mailStrip']),
    ];
  }

  public function getColor(?string $any_string): string {
    if(is_null($any_string)) {
      return 'Blue';
    }
    $colors = [
      "A" => [
        "Almond",
      ],
      "B" => ["Beige",],
      "C" => [
        "Carmine",
      ],
      "D" => ["Denim"],
      "E" => ["Emerald"],
      "F" => ["Fuchsia"],
      "G" => ["Green"],
      "H" => ["Harlequin"],
      "I" => ["Indigo", ],
      "J" => ["Jasmine"],
      "K" => ["Khaki"],
      "L" => ["Lavender",],
      "M" => ["Magenta",],
      "N" => ["Navy"],
      "O" => ["Ochre",],
      "P" => [
        "Purple",
      ],
      "R" => ["Red",],
      "S" => ["Silver"],
      "T" => [ "Turquoise"],
      "U" => ["Ultramarine"],
      "V" => ["Violet"],
      "W" => ["White"],
      "Y" => ["Yellow"],
    ];

    $first_letter = strtoupper(substr($any_string, 0, 1));
    $picked_color = $colors[$first_letter];
    $index = array_rand($picked_color);
    return $picked_color[$index];
  }

  public function isUnseen(mixed $any_string): bool {
    return !empty(trim($any_string));
  }

  public function isFlagged(mixed $any_string): bool {
    return !empty(trim($any_string));
  }

  public function isRecent(mixed $any_string): bool {
    return !empty(trim($any_string));
  }

  public function json(mixed $any): string {
    return json_encode($any, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
  }

  public function getIcon(string $box_name): string {
    $box_name = strtolower($box_name);
    $remixIcons = [
      'mailboxes' => [
        'inbox' => 'ri-inbox-line', // Inbox
        'sent' => 'ri-send-plane-line', // Sent
        'drafts' => 'ri-file-list-2-line', // Drafts
        'trash' => 'ri-delete-bin-line', // Trash
        'spam' => 'ri-error-warning-line', // Spam/Junk
        'archive' => 'ri-archive-line', // Archive
        'outbox' => 'ri-send-plane-2-line', // Outbox
        'all_mail' => 'ri-mail-line', // All Mail
        'important' => 'ri-star-line', // Important
        'starred' => 'ri-star-fill', // Starred
        'starred_mail' => 'ri-star-half-line', // Starred Mail
        'junk' => 'ri-error-warning-line',
        'default_folder' => 'ri-folder-line',
      ],
      'labels' => [
        'work' => 'ri-briefcase-line', // Work
        'personal' => 'ri-user-line', // Personal
        'finance' => 'ri-wallet-line', // Finance
        'shopping' => 'ri-shopping-cart-line', // Shopping
        'social' => 'ri-share-box-line', // Social
        'notifications' => 'ri-notification-line', // Notifications
        'projects' => 'ri-folder-line', // Projects
        'follow_up' => 'ri-checkbox-circle-line', // Follow Up
        'urgent' => 'ri-alert-line', // Urgent
      ],
    ];
     return  $remixIcons['mailboxes'][$box_name] ?? $remixIcons['labels'][$box_name] ?? $remixIcons['mailboxes']['default_folder'];
  }

  public function getSize(mixed $any_string): string {
    $length = strlen($any_string);

    if ($length < 1024) {
      return $length . ' bytes';  // Size in bytes
    }

    $kb = $length / 1024;
    if ($kb < 1024) {
      return number_format($kb, 2) . ' KB';  // Size in kilobytes
    }

    $mb = $kb / 1024;
    if ($mb < 1024) {
      return number_format($mb, 2) . ' MB';  // Size in megabytes
    }

    $gb = $mb / 1024;
    if ($gb < 1024) {
      return number_format($gb, 2) . ' GB';  // Size in gigabytes
    }

    $tb = $gb / 1024;
    return number_format($tb, 2) . ' TB';  // Size in terabytes
  }

  public function getActions(): array {
    return [
      Flag::FLAGGED => 'Flagged',
      Flag::UNFLAGGED => 'Unflagged',
      Flag::SEEN => 'Seen',
      Flag::UNSEEN => 'Unseen',
    ];
  }

  public function mailStrip(string|null $any_string): ?string {
    if(is_null($any_string)) {
      return NULL;
    }
    $start = strpos($any_string, '<');
    $end = strpos($any_string, '>');

    if ($start !== false && $end !== false) {
      return substr($any_string, $start + 1, $end - $start - 1);
    }
    return $any_string;
  }
}
